/*
 * Copyright (C) 1999-2020 Hartmut Seichter
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef PW_SCENE_ENTITY_HPP
#define PW_SCENE_ENTITY_HPP

#include <pw/core/globals.hpp>
#include <pw/scene/scene.hpp>

#include <string>
#include <vector>
#include <memory>

#include <entt/entt.hpp>

namespace pw {

class entity {
public:
    entity() = default;
    entity(const entity&) = default;
    entity(scene& s);
    ~entity();

    template <typename T,typename... Args>
    T& add_component(Args&&... args) {
        return _registry->emplace<T>(this->_entity,std::forward<Args>(args)...);
    }

    template <typename T>
    void remove_component(entity) {
        _registry->remove<T>(_entity);
    }

    template<typename T>
    T& get_component() const {
        return _registry->get<T>(_entity);
    }

    template <typename T,typename... Args>
    T& get_or_create_component(Args&&... args) {
        return _registry->get_or_emplace<T>(this->_entity,std::forward<Args>(args)...);
    }

    template<typename T>
    bool has_component() const { return _registry->valid(_entity);}

    void destroy();

    operator bool() const { return _registry && _entity != entt::null; }

    operator std::uint32_t() const { return static_cast<std::uint32_t>(_entity); }

    bool valid() const { return _registry->valid(_entity); }


    void add_child(entity &child);

    void remove_child(entity &child);

    size_t child_count() const;


private:

    std::shared_ptr<entt::registry> _registry;
    entt::entity _entity { entt::null };

};

}
#endif
