#include "pw/scene/entity.hpp"
#include "pw/scene/scene.hpp"

#include "pw/core/debug.hpp"

namespace pw {

struct relationship {
    std::vector<entt::entity> children;
    entt::entity parent { entt::null };
};

entity::entity(scene &s)
    : _registry(s._registry)
    , _entity(s._registry ? _registry->create() : entt::null)
{
}

void entity::add_child(entity& child)
{
    if (child._registry != this->_registry) {
        debug::e() << "Cannot mix entities from different scenes";
        return;
    }

    // TODO: check circular dependencies

    // declare child relationship
    auto r = get_or_create_component<relationship>();
    r.children.push_back(child._entity);

    // declare parent
    auto p_r = child.get_or_create_component<relationship>();
    p_r.parent = _entity;
}

void entity::remove_child(entity& child)
{
    if (child._registry != this->_registry) {
        debug::e() << "Cannot mix entities from different scenes";
        return;
    }

    // both need to have a relationship component
    if (has_component<relationship>() && child.has_component<relationship>())
    {
        // we need to check if the child is related to the parent
        auto r_p = child.get_component<relationship>();
        if (r_p.parent == _entity)
        {
            // now go ahead
            r_p.parent = entt::null;

            auto r = get_component<relationship>();
            std::remove(r.children.begin(),r.children.end(),child._entity);

        }
    }
}

size_t entity::child_count() const
{
    return has_component<relationship>() ? get_component<relationship>().children.size() : 0;
}

entity::~entity()
{
    this->destroy();
}

void entity::destroy()
{
    _registry->destroy(_entity);
}

}

